mMinimap = LibStub("AceAddon-3.0"):NewAddon("mMinimap", "AceEvent-3.0", "AceConsole-3.0")
local mMinimap = mMinimap

-- Locals
local db
local OptionsFrame

-- GUI
local UIF
local LSM = LibStub("LibSharedMedia-3.0")
local AceGUI = LibStub("AceGUI-3.0")

local mMinimapIcon = [[Interface\Icons\INV_Offhand_Sunwell_D_01]]
local MMINIMAP_ANCHOR_POINTS = {
	CENTER = "CENTER",
	TOPLEFT = "TOPLEFT",
	TOPRIGHT = "TOPRIGHT",
	BOTTOMLEFT = "BOTTOMLEFT",
	BOTTOMRIGHT = "BOTTOMRIGHT",
}

mMinimap.WidgetRegistry = AceGUI.WidgetRegistry
mMinimap.WidgetVersions = AceGUI.WidgetVersions
mMinimap.WidgetPools = AceGUI.objPools

function mMinimap:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			MoveLFDFrame = true,
			HideTracking = true,
			MinimapLocked = true,
			MinimapScale = 1,
			MinimapPositionX = 0,
			MinimapPositionY = 0,
			MinimapAnchor = "UIParent",
			MinimapAnchorPoint = "CENTER",
			MinimapAnchorRelativePoint = "CENTER",
			MinimapShape = "SQUARE",
			
			-- Font
			ZoneFont = "accid",
			ZoneFontSize = 16,
			ZoneFontFlags = "OUTLINE",
			ZoneTextX = 0,
			ZoneTextY = 3,
		}
	}
	
	self.db  = LibStub("AceDB-3.0"):New("mMinimapDB", defaults, "Default")
	db = self.db.profile
	
	self:SetupLDB()
	
	self:SetEnabledState(db.Enabled)
end

function mMinimap:OnEnable()
	
	-- Register Events
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "NewArea")
	-- Register slash command
	self:RegisterChatCommand("mm", "ToggleConfig")
	
	local UIFSettings = {
		s = {
			font = LibStub("LibSharedMedia-3.0"):Fetch("font", "accid"),
			fontSize = 16,			
		}
	}

	UIF = LibStub("GUI-Factory-1.0"):GetFactory("mMinimap", UIFSettings)
	
	self:ChangeCallback()
end

function mMinimap:OnDisable()
end


function mMinimap:HideDefaultFrames()
	-- Mousewheel zoom
	MinimapZoomIn:Hide()
	MinimapZoomOut:Hide()
	Minimap:EnableMouseWheel()
	Minimap:SetScript('OnMouseWheel', function(self, direction)
		if(direction > 0) then
			MinimapZoomIn:Click()
		else
			MinimapZoomOut:Click()
		end
	end)
	
	-- Tracking
	if db.HideTracking then
		MiniMapTracking:Hide()
	else
		MiniMapTracking:Show()		
	end
	
	-- LFG
	MiniMapLFGFrameBorder:Hide()	
	
	-- Border
	MinimapBorder:SetTexture(nil)
	MinimapBorderTop:Hide()
	
	-- Compas
	MinimapNorthTag:SetAlpha(0)
	
	-- Instance Difficulty
	MiniMapInstanceDifficulty:Hide()
end


function mMinimap:MoveFrames()
	Minimap:ClearAllPoints()
	local scale = db.MinimapScale
	Minimap:SetPoint("CENTER", db.MinimapAnchor, "CENTER", db.MinimapPositionX / scale, db.MinimapPositionY / scale)
	Minimap:SetScale(scale)
	
	-- Tracking
	MiniMapTracking:ClearAllPoints()
	MiniMapTracking:SetPoint("TOPLEFT", Minimap, "TOPLEFT")		
		
	-- Looking for Dungeon (Tooltip)
	if db.MoveLFDFrame then
		LFDSearchStatus:ClearAllPoints()
		LFDSearchStatus:SetPoint("CENTER", UIParent)
	else
		LFDSearchStatus:ClearAllPoints()
		LFDSearchStatus:SetPoint("TOPRIGHT", MiniMapLFGFrame, "TOPLEFT")
	end
	
	-- Looking for Dungeon (Icon)
	MiniMapLFGFrame:ClearAllPoints()
	MiniMapLFGFrame:SetPoint("TOP", MiniMapTracking, "BOTTOM")
	
	-- Battleground
	MiniMapBattlefieldFrame:ClearAllPoints()
	MiniMapBattlefieldFrame:SetPoint("TOPRIGHT", Minimap)
	MiniMapBattlefieldBorder:SetTexture(nil)
	
	-- New Mail
	MiniMapMailFrame:ClearAllPoints()
	MiniMapMailFrame:SetParent(Minimap)
	MiniMapMailFrame:SetPoint("TOP", 0, -4)
	MiniMapMailFrame:SetHeight(8)
	MiniMapMailBorder:SetTexture(nil)
	
	-- World map button
	MiniMapWorldMapButton:Hide()
	
	-- Time/Calendar
	GameTimeFrame:Hide()
	
	-- Zone text
	MinimapZoneText:SetAllPoints(MinimapZoneTextButton)
	MinimapZoneTextButton:ClearAllPoints()
	MinimapZoneTextButton:SetParent(Minimap)
	MinimapZoneTextButton:SetPoint("BOTTOM", Minimap, "TOP", db.ZoneTextX, db.ZoneTextY)
	MinimapZoneTextButton:SetWidth(Minimap:GetWidth() * 1.5)
	
	-- Instance Difficulty
	MiniMapInstanceDifficulty:ClearAllPoints()
	MiniMapInstanceDifficulty:SetPoint("BOTTOMRIGHT", Minimap)
end


function mMinimap:SetAttributes()

	-- Minimap Shape
	if db.MinimapShape == "SQUARE" then
		Minimap:SetMaskTexture([=[Interface\ChatFrame\ChatFrameBackground]=])
	else
		Minimap:SetMaskTexture([=[Textures\MinimapMask]=])
	end
	
	MiniMapTrackingButtonBorder:Hide()
	MiniMapTrackingIcon:SetTexCoord(0.065, 0.935, 0.065, 0.935)
	
	-- Movable Minimap
	if not db.MinimapLocked then
		Minimap:RegisterForDrag("LeftButton")
		Minimap:SetMovable(true)
		Minimap:SetScript('OnDragStop', function() 
			Minimap:StopMovingOrSizing() 
			local scale = Minimap:GetEffectiveScale() / UIParent:GetEffectiveScale()
			
			x, y = Minimap:GetCenter()
			x, y = x * scale, y * scale
			
			x = x - GetScreenWidth() / 2
			y = y - GetScreenHeight() / 2
			
			db.MinimapPositionX, db.MinimapPositionY = x, y
			
			if OptionsFrame then
				OptionsFrame:ReleaseChildren()
				OptionsFrame:Release()
				OptionsFrame = nil
				mMinimap:ToggleConfig()
			end
		end)
		Minimap:SetScript('OnDragStart', function() 
			Minimap:StartMoving() 
		end)
		MinimapCluster:EnableMouse(false)
		Minimap:SetClampedToScreen(true)
	else
		Minimap:SetMovable(false)
		Minimap:SetScript("OnDragStop", nil)
		Minimap:SetScript("OnDragStart", nil)
	end
	
	-- Zone Text
	MinimapZoneText:SetFont(LSM:Fetch("font", db.ZoneFont), db.ZoneFontSize, db.ZoneFontFlags)
end

---------------------------------------------------
-- Events
---------------------------------------------------
Minimap:SetScript("OnMouseUp", function(self, button)
	if (button == 'RightButton') and  db.HideTracking then
		ToggleDropDownMenu(1, nil, MiniMapTrackingDropDown, self, 0, 0)
	elseif (button == "MiddleButton") then
		ToggleCalendar()
	else
		Minimap_OnClick(self, button)
	end
end)

Minimap:SetScript("OnEnter", function()
	MiniMapTracking:Show()
	MiniMapInstanceDifficulty:Show()
end)

Minimap:SetScript("OnLeave", function()
	mMinimap:HideDefaultFrames()
	MiniMapInstanceDifficulty:Hide()
end)

function mMinimap:NewArea()
end

---------------------------------------------------
-- Config
---------------------------------------------------
do	
	function mMinimap:ChangeCallback(widget, event, ...)
		if widget and not widget.GetUserData then
			widget = self
			self = mMinimap
		end		
		self:HideDefaultFrames()
		self:MoveFrames()
		self:SetAttributes()
	end

	function mMinimap:ToggleConfig()
		if not OptionsFrame then	
			local m = UIF:Frame("mMinimap")
			m:SetLayout("Fill")
			m:SetWidth(810)
			m:SetHeight(620)
			m:SetCallback("OnClose", function(widget)
				widget:ReleaseChildren()
				widget:Release()
				OptionsFrame = nil
			end)				
			m:Show()			
			OptionsFrame = m
			
			m = AceGUI:Create("ScrollFrame")
			m:SetLayout("Flow")
			OptionsFrame:AddChild(m)
			
			local select, slider, color, group, group2, check, text
			group = UIF:InlineGroup("")
			text = UIF:NormalText([[- Right click will show the tracking menu.
- Middle click shows the calendar.
- Hover over to show the clock (and tracking icon if it is hidden).]])
			text:SetImage(mMinimapIcon)
			group:AddChild(text)
			m:AddChild(group)
			
			m:AddChild(UIF:CheckBox("Lock Minimap", db, "MinimapLocked", mMinimap.ChangeCallback, "While locked, the Minimap cannot be moved with the mouse."))
			m:AddChild(UIF:CheckBox("Move \"Looking For Fungeon\" tooltip", db, "MoveLFDFrame", mMinimap.ChangeCallback, "This will move the tooltip to the middle of the screen"))
			m:AddChild(UIF:CheckBox("Hide tracking icon", db, "HideTracking", mMinimap.ChangeCallback, "If hidden right click will show the tracking menu."))
			
			m:AddChild(UIF:Slider("Scale", db, "MinimapScale", 0.01, 10, 0.01, mMinimap.ChangeCallback))
			
			m:AddChild(UIF:Slider("X", db, "MinimapPositionX", 
				-floor(GetScreenWidth() * UIParent:GetEffectiveScale()), 
				floor(GetScreenWidth() * UIParent:GetEffectiveScale()), 0.1, mMinimap.ChangeCallback))
			m:AddChild(UIF:Slider("Y", db, "MinimapPositionY", 
				-floor(GetScreenHeight() * UIParent:GetEffectiveScale()), 
				floor(GetScreenHeight() * UIParent:GetEffectiveScale()), 0.1, mMinimap.ChangeCallback))
				
			select = UIF:Dropdown("Minimap Shape", db, "MinimapShape", mMinimap.ChangeCallback, {SQUARE = "Square", ROUND = "Round"})
			select:SetValue(db.MinimapShape)
			m:AddChild(select)
			
			group = UIF:InlineGroup("Zone Text")
			group:AddChild(UIF:LSMDropdown("font", "Font", db, "ZoneFont", mMinimap.ChangeCallback))
			group:AddChild(UIF:Slider("Font Size", db, "ZoneFontSize", 1, 50, 1, mMinimap.ChangeCallback))
			group:AddChild(UIF:Slider("Offset X", db, "ZoneTextX", -50, 50, 0.1, mMinimap.ChangeCallback))
			group:AddChild(UIF:Slider("Offset Y", db, "ZoneTextY", -200, 200, 0.1, mMinimap.ChangeCallback))
			m:AddChild(group)
		end
	end

	function mMinimap:SetupLDB()
		local LDB = LibStub("LibDataBroker-1.1")
		if not LDB then return end
		
		local l = LDB:NewDataObject("mMinimap")
		l.type = "launcher"
		l.icon = mMinimapIcon
		l.tocname = "mMinimap"
		l.label = "mMinimap"
		l.OnClick = function(self, button)
			mMinimap:ToggleConfig()
		end
		l.OnTooltipShow = function(tt)
			tt:AddLine("mMinimap")
			tt:AddLine("Click to toggle configuration")
		end
		
		self.ldbojb = l
	end
end

function GetMinimapShape() return db.MinimapShape end